% Parameter fitting routine for ODE model of BPM
% E contains the endpoints and the error
% S contains the startpoints, the final err and the initial err
% a row of S is: [err InitErr randTheta']
% history is every point visited and the error

function [E S history] = fitParams(iter, ModName, X0, N, Data, ...
                                   prior, priorH, maxIterFit, ...
                                   numSavErr, numDec, options, poolSize)
    
    % R keeps the final error and the parameters 
    % for every iteration.
    % S keeps the initial error and parameters 
    % for every iteration
    R = zeros(maxIterFit, N+1);
    S = zeros(maxIterFit, N+2); 
       
    % Set parallel jobs
    try,
        matlabpool close
    end
    fprintf('Creating %i workers...', poolSize);
    matlabpool(poolSize);
    fprintf(' done.\n\nStarting sampling:\n\n');
    
    % Give every worker its own seed for rand
    spmd
        newSeed = sum(100*( 1 + labindex/numlabs)*clock);
        %RandStream.setDefaultStream(RandStream('mt19937ar', ...
        %          'seed',sum(100*( 1 + labindex/numlabs)*clock)));
        rand('state', newSeed);
    end
    
    % Parallel iteration
    parfor j=1:maxIterFit
        % Get random params
        randTheta = getRandParams(prior, priorH);
                
        % Nelder-Mead optimisation
        % fminsearch is set up so it minimises objFun only on theta
        % the other variables are considered parameters and are fixed
        [params err, fl, outpt] = ...
            fminsearch(@(theta) objFun(ModName, theta, X0, Data, numDec), ...
                       randTheta, options);
        
        % Without saving all history, to be deprecated
        hst = [];
        
        % Get error with the initial theta
        initErr = objFun(ModName, randTheta, X0, Data, numDec);
        
        % Store results
        R(j, :) = [err params'];
        S(j, :) = [err initErr randTheta'];
        fprintf('\n%i.-\tInit Err: %f\tFinal Error: %f\t\t(Run %i)', ...
                j, initErr, err, iter);
    end
    try,
        matlabpool close
    end
    
    errMin = min(R(:,1));
    iMin = find(R(:,1)==errMin);
    
    % Sort R and S so we can get E
    E = sortrows(R);
    S = sortrows(S);
    
    %E = R(1:numSavErr, :);
    %S = S(1:numSavErr, :);
    
    history = [];
    
    fprintf('\n\nThe minimum error was:\t%f\t(iteration %i)\n', ...
            errMin, iMin)
    
end


    
    

    
    
    
    
    
    
    
    
    
    
    
    